# =====================================================================
# (C) 2019-2021 Tech Mahindra Limited. All rights reserved
# No part of this code may be copied, reproduced, sold,
# or transmitted in any form or by any means, electronic,
# mechanical, photocopying, recording, or otherwise,
# without prior written permission of Tech Mahindra Limited.
#
# IN NO EVENT SHALL TECH MAHINDRA BE LIABLE TO ANY PARTY FOR DIRECT,
# INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE
# AND ITS DOCUMENTATION, EVEN IF TECH MAHINDRA HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
# TECH MAHINDRA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
# DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS".
# TECH MAHINDRA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
# UPDATES, ENHANCEMENTS OR MODIFICATIONS.
# =====================================================================

from novalinkAutoInstaller.src import NovalinkTemplateGen
from novalinkAutoInstaller.src import NovalinkInstaller
from novalinkAutoInstaller.src import NovalinkConfiguration
import sys
import argparse
import getpass


def validateRepetitionArgs(optionValue, argsLength):
    if argsLength > 1:
        print(str(optionValue)+" appears several times")
        exit(1)


def addParserAgrs(parser):
    parser.add_argument(
        "-i",  action='append', required=True, help="Specifies the IP \
            address. HMC IP for Generate/Install")
    parser.add_argument(
        "-u",  action='append', required=True, help="Specifies the Username. \
            HMC Username for Generate/Install")
    parser.add_argument(
        "-m", action='append', help="The name of \
            the host on which to perform the operation")
    parser.add_argument(
        "-n", type=int, action='append', choices=[
            1, 2, 3, 4, 5, 6, 7, 8], help="Specifies the \
                numbers of VIOS to be created.Valid values are between 1 to 8")
    parser.add_argument(
        "-a", action='append',  help="(Optional) Specifies \
            SRIOV adapter mode. Supported values are sriov or dedicated")
    parser.add_argument(
        "-r", action='append',  help="(Optional) Specifies \
            Novalink Installation Mode. Valid value is normal")
    parser.add_argument(
        "-hostname", action='append',  help="Specifies \
            the Hostname for NovaLink")
    parser.add_argument(
        "--generate",  dest='generate', action='store_true', help="Specifies \
            Novalink Installer template generation")
    parser.add_argument(
        "--install",   dest='install', help="Specifies \
            Novalink Installation", action='store_true')
    parser.add_argument(
        "--accept-license",  dest='acceptlicense', help="\
            Accept License", action='store_true')
    parser.add_argument(
        "--configure",  dest='configure', help="\
            Configure NovaLink", action='store_true')


def generate(args):
    # if not args.r:
    #   print("Error: argument -r is required")
    #   exit(1)
    if args.m:
        validateRepetitionArgs('-m', len(args.m))
        frameName = args.m[0]
        genTemplate = True
    else:
        print("Error: argument -m is required")
        exit(1)
    # validating numeric values for number of VIOS (-n) option
    if args.n:
        validateRepetitionArgs('-n', len(args.n))
        try:
            noOfVios = int(args.n[0])
        except Exception as e:
            print("Invalid entry for -n option. \
                Please enter a numeric value")
            exit(1)
    else:
        print("Error: argument -n is required")
        exit(1)
    return frameName, noOfVios, genTemplate


def install(args):
    # if not args.r:
    #    print("Error: argument -r is required")
    #    exit(1)
    if args.m:
        print("Error: can not specify --install and -m flags together.")
        exit(1)
    if args.n:
        print("Error: can not specify --install and -n flags together.")
        exit(1)
    if not args.acceptlicense:
        print("Error: argument --accept-license is required")
        exit(1)
    return True


def configure(args):
    if args.m:
        print("Error: can not specify --configure and -m flags together.")
        exit(1)
    if args.n:
        print("Error: can not specify --configure and -n flags together.")
        exit(1)
    if args.r:
        print("Error: can not specify --configure and -r flags together.")
        exit(1)


def validate_input(args):
    if not args.generate and not args.install and not args.configure:
        print(
            "Error: argument --generate, --install or --configure is required")
        exit(1)
    if args.i:
        validateRepetitionArgs('-i', len(args.i))
        ipAddress = args.i[0]
    if args.u:
        validateRepetitionArgs('-u', len(args.u))
        userName = args.u[0]
    # validating dedicated string values for
    # installation mode (-r) option
    if args.r:
        validateRepetitionArgs('-r', len(args.r))
        mode = args.r[0].lower()
        if mode != "normal":
            print("Invalid entry for Installation mode. \
                Valid modes is normal")
            exit(1)
    else:
        mode = 'normal'
    if args.a:
        validateRepetitionArgs('-a', len(args.a))
        sriov = args.a[0].lower()
        if sriov != "sriov" and sriov != "dedicated":
            print("Invalid entry for SRIOV adapter mode. \
                Valid values are sriov or dedicated")
            exit(1)
    else:
        sriov = 'sriov'

    return ipAddress, userName, mode, sriov


# Main Function
def main():
    genTemplate = False
    installNovalink = False
    parser = argparse.ArgumentParser()
    addParserAgrs(parser)
    # In case of no cli argument pass, printing help messages
    if len(sys.argv) <= 1:
        print(parser.print_help())
        exit(1)
    args = parser.parse_args()
    (ipAddress, userName, mode, sriov) = validate_input(args)
    if args.generate and args.install:
        print("Error: can not specify --generate and --install flags together")
        exit(1)
    if args.generate and args.configure:
        print(
            "Error: can not specify --generate and --configure flags together")
        exit(1)
    if args.configure and args.install:
        print(
            "Error: can not specify --configure and --install flags together")
        exit(1)
    elif args.generate:
        frameName, noOfVios, genTemplate = generate(args)
    elif args.install:
        installNovalink = install(args)
    elif args.configure:
        configure(args)
        NLHostname = False
        if args.hostname:
            NLHostname = args.hostname[0]
        else:
            NLHostname = 'novalink-'+ipAddress.replace(".", "-")
        NovalinkConfiguration.main(ipAddress, userName, NLHostname)
        exit(0)
    hmcPassword = getpass.getpass(prompt="HMC Password:")
    if genTemplate:
        NovalinkTemplateGen.main(ipAddress, userName, hmcPassword, str(
            frameName), noOfVios, mode, sriov)
    elif installNovalink:
        NovalinkInstaller.main(ipAddress, userName, hmcPassword, mode)


# Main Function
if __name__ == "__main__":
    sys.exit(main())
